@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,8 +36,14 @@
 # include "tag_editor.h"
 #endif // HAVE_TAGLIB_H
 
-using namespace Global;
-using namespace MPD;
+using Global::MainHeight;
+using Global::MainStartY;
+using Global::myScreen;
+using Global::RedrawHeader;
+
+using MPD::itDirectory;
+using MPD::itSong;
+using MPD::itPlaylist;
 
 Browser *myBrowser = new Browser;
 
@@ -51,9 +57,10 @@ const char *Browser::SupportedExtensions[] =
 
 void Browser::Init()
 {
-	w = new Menu<Item>(0, MainStartY, COLS, MainHeight, Config.columns_in_browser ? Display::Columns() : "", Config.main_color, brNone);
+	w = new Menu<MPD::Item>(0, MainStartY, COLS, MainHeight, Config.columns_in_browser ? Display::Columns() : "", Config.main_color, brNone);
 	w->HighlightColor(Config.main_highlight_color);
 	w->CyclicScrolling(Config.use_cyclic_scrolling);
+	w->CenteredCursor(Config.centered_cursor);
 	w->SetSelectPrefix(&Config.selected_item_prefix);
 	w->SetSelectSuffix(&Config.selected_item_suffix);
 	w->SetItemDisplayer(Display::Items);
@@ -89,7 +96,7 @@ void Browser::SwitchTo()
 	w->Empty() ? myBrowser->GetDirectory(itsBrowsedDir) : myBrowser->UpdateItemList();
 
 	if (myScreen != this && myScreen->isTabbable())
-		myPrevScreen = myScreen;
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	RedrawHeader = 1;
 }
@@ -97,7 +104,7 @@ void Browser::SwitchTo()
 std::basic_string<my_char_t> Browser::Title()
 {
 	std::basic_string<my_char_t> result = U("Browse: ");
-	result += Scroller(TO_WSTRING(itsBrowsedDir), itsScrollBeginning, w->GetWidth()-result.length()-(Config.new_design ? 2 : VolumeState.length()));
+	result += Scroller(TO_WSTRING(itsBrowsedDir), itsScrollBeginning, w->GetWidth()-result.length()-(Config.new_design ? 2 : Global::VolumeState.length()));
 	return result;
 }
 
@@ -106,7 +113,7 @@ void Browser::EnterPressed()
 	if (w->Empty())
 		return;
 	
-	const Item &item = w->Current();
+	const MPD::Item &item = w->Current();
 	switch (item.type)
 	{
 		case itDirectory:
@@ -122,7 +129,7 @@ void Browser::EnterPressed()
 		}
 		case itPlaylist:
 		{
-			SongList list;
+			MPD::SongList list;
 			Mpd.GetPlaylistContent(locale_to_utf_cpy(item.name), list);
 			if (myPlaylist->Add(list, 1))
 				ShowMessage("Loading and playing playlist %s...", item.name.c_str());
@@ -147,7 +154,7 @@ void Browser::SpacePressed()
 	if (itsBrowsedDir != "/" && w->Choice() == 0 /* parent dir */)
 		return;
 	
-	const Item &item = w->Current();
+	const MPD::Item &item = w->Current();
 	switch (item.type)
 	{
 		case itDirectory:
@@ -155,11 +162,11 @@ void Browser::SpacePressed()
 			if (itsBrowsedDir != "/" && !w->Choice())
 				break; // do not let add parent dir.
 			
-			SongList list;
+			MPD::SongList list;
 #			ifndef WIN32
 			if (isLocal())
 			{
-				ItemList items;
+				MPD::ItemList items;
 				ShowMessage("Scanning \"%s\"...", item.name.c_str());
 				myBrowser->GetLocalDirectory(items, item.name, 1);
 				list.reserve(items.size());
@@ -183,7 +190,7 @@ void Browser::SpacePressed()
 		}
 		case itPlaylist:
 		{
-			SongList list;
+			MPD::SongList list;
 			Mpd.GetPlaylistContent(locale_to_utf_cpy(item.name), list);
 			if (myPlaylist->Add(list, 0))
 				ShowMessage("Loading playlist %s...", item.name.c_str());
@@ -247,13 +254,15 @@ void Browser::ReverseSelection()
 
 void Browser::GetSelectedSongs(MPD::SongList &v)
 {
+	if (w->Empty())
+		return;
 	std::vector<size_t> selected;
 	w->GetSelected(selected);
 	if (selected.empty())
 		selected.push_back(w->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 	{
-		const Item &item = w->at(*it);
+		const MPD::Item &item = w->at(*it);
 		switch (item.type)
 		{
 			case itDirectory:
@@ -273,7 +282,7 @@ void Browser::GetSelectedSongs(MPD::SongList &v)
 			}
 			case itSong:
 			{
-				v.push_back(new Song(*item.song));
+				v.push_back(new MPD::Song(*item.song));
 				break;
 			}
 			case itPlaylist:
@@ -315,13 +324,11 @@ void Browser::LocateSong(const MPD::Song &s)
 	if (myScreen != this)
 		SwitchTo();
 	
-	std::string option = s.toString(Config.song_status_format);
-	locale_to_utf(option);
 	if (itsBrowsedDir != s.GetDirectory())
 		GetDirectory(s.GetDirectory());
 	for (size_t i = 0; i < w->Size(); ++i)
 	{
-		if (w->at(i).type == itSong && option == w->at(i).song->toString(Config.song_status_format))
+		if ((*w)[i].type == itSong && s.GetHash() == (*w)[i].song->GetHash())
 		{
 			w->Highlight(i);
 			break;
@@ -350,16 +357,16 @@ void Browser::GetDirectory(std::string dir, std::string subdir)
 	
 	if (dir != "/")
 	{
-		Item parent;
+		MPD::Item parent;
 		size_t slash = dir.rfind("/");
-		parent.song = reinterpret_cast<Song *>(1); // in that way we assume that's really parent dir
+		parent.song = reinterpret_cast<MPD::Song *>(1); // in that way we assume that's really parent dir
 		parent.name = slash != std::string::npos ? dir.substr(0, slash) : "/";
 		parent.type = itDirectory;
 		utf_to_locale(parent.name);
 		w->AddOption(parent);
 	}
 	
-	ItemList list;
+	MPD::ItemList list;
 #	ifndef WIN32
 	isLocal() ? GetLocalDirectory(list) : Mpd.GetDirectory(dir, list);
 #	else
@@ -368,7 +375,7 @@ void Browser::GetDirectory(std::string dir, std::string subdir)
 	if (!isLocal()) // local directory is already sorted
 		sort(list.begin(), list.end(), CaseInsensitiveSorting());
 	
-	for (ItemList::iterator it = list.begin(); it != list.end(); ++it)
+	for (MPD::ItemList::iterator it = list.begin(); it != list.end(); ++it)
 	{
 		switch (it->type)
 		{
@@ -407,7 +414,7 @@ void Browser::GetDirectory(std::string dir, std::string subdir)
 }
 
 #ifndef WIN32
-void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool recursively) const
+void Browser::GetLocalDirectory(MPD::ItemList &v, const std::string &directory, bool recursively) const
 {
 	DIR *dir = opendir((directory.empty() ? itsBrowsedDir : directory).c_str());
 	
@@ -419,23 +426,16 @@ void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool
 	struct stat file_stat;
 	std::string full_path;
 	
-	// omit . and ..
-	for (int i = 0; i < 2; ++i)
-	{
-		file = readdir(dir);
-		if (!file)
-		{
-			closedir(dir);
-			return;
-		}
-	}
-	
 	size_t old_size = v.size();
 	while ((file = readdir(dir)))
 	{
+		// omit . and ..
+		if (file->d_name[0] == '.' && (file->d_name[1] == '\0' || (file->d_name[1] == '.' && file->d_name[2] == '\0')))
+			continue;
+		
 		if (!Config.local_browser_show_hidden_files && file->d_name[0] == '.')
 			continue;
-		Item new_item;
+		MPD::Item new_item;
 		full_path = directory.empty() ? itsBrowsedDir : directory;
 		if (itsBrowsedDir != "/")
 			full_path += "/";
@@ -459,7 +459,7 @@ void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool
 		{
 			new_item.type = itSong;
 			mpd_pair file_pair = { "file", full_path.c_str() };
-			new_item.song = new Song(mpd_song_begin(&file_pair));
+			new_item.song = new MPD::Song(mpd_song_begin(&file_pair));
 #			ifdef HAVE_TAGLIB_H
 			if (!recursively)
 				TagEditor::ReadTags(*new_item.song);
@@ -481,19 +481,12 @@ void Browser::ClearDirectory(const std::string &path) const
 	struct stat file_stat;
 	std::string full_path;
 	
-	// omit . and ..
-	for (int i = 0; i < 2; ++i)
-	{
-		file = readdir(dir);
-		if (!file)
-		{
-			closedir(dir);
-			return;
-		}
-	}
-	
 	while ((file = readdir(dir)))
 	{
+		// omit . and ..
+		if (file->d_name[0] == '.' && (file->d_name[1] == '\0' || (file->d_name[1] == '.' && file->d_name[2] == '\0')))
+			continue;
+		
 		full_path = path;
 		if (*full_path.rbegin() != '/')
 			full_path += '/';
