@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -29,8 +29,7 @@ namespace Global
 {
 	extern BasicScreen *myScreen;
 	extern BasicScreen *myOldScreen;	// for info, lyrics, popups
-	extern BasicScreen *myPrevScreen;	// "real" screen switching
-						// 	(browser, search, etc.)
+	extern BasicScreen *myPrevScreen;	// "real" screen switching (browser, search, etc.)
 	
 	extern Window *wHeader;
 	extern Window *wFooter;
