@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,7 +25,8 @@
 #include "settings.h"
 #include "tag_editor.h"
 
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
 
 Help *myHelp = new Help;
 
@@ -46,6 +47,8 @@ void Help::Resize()
 
 void Help::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -56,9 +59,9 @@ void Help::SwitchTo()
 		Resize();
 	
 	if (myScreen != this && myScreen->isTabbable())
-		myPrevScreen = myScreen;
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 
 }
 
@@ -73,7 +76,7 @@ std::string Help::DisplayKeys(int *key, int size)
 	std::string result = "\t";
 	for (int i = 0; i < size; i++)
 	{
-		if (key[i] == null_key);
+		if (key[i] == NcmpcppKeys::NullKey);
 		else if (key[i] == 259)
 			result += "Up";
 		else if (key[i] == 258)
@@ -130,15 +133,19 @@ void Help::GetKeybindings()
 	*w << "   " << fmtBold << "Keys - Movement\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Up)			<< "Move Cursor up\n";
 	*w << DisplayKeys(Key.Down)			<< "Move Cursor down\n";
+	*w << DisplayKeys(Key.UpAlbum)			<< "Move Cursor up one album\n";
+	*w << DisplayKeys(Key.DownAlbum)		<< "Move Cursor down one album\n";
+	*w << DisplayKeys(Key.UpArtist)			<< "Move Cursor up one artist\n";
+	*w << DisplayKeys(Key.DownArtist)		<< "Move Cursor down one artist\n";
 	*w << DisplayKeys(Key.PageUp)			<< "Page up\n";
 	*w << DisplayKeys(Key.PageDown)			<< "Page down\n";
 	*w << DisplayKeys(Key.Home)			<< "Home\n";
 	*w << DisplayKeys(Key.End)			<< "End\n";
 	*w << "\n";
-	if (Config.screen_switcher_browser_only)
-		*w << DisplayKeys(Key.ScreenSwitcher)   << "Switch between playlist and browser\n";
-	else
+	if (Config.screen_switcher_previous)
 		*w << DisplayKeys(Key.ScreenSwitcher)   << "Switch between current and last screen\n";
+	else
+		*w << DisplayKeys(Key.ScreenSwitcher)   << "Switch between given sequence of screens\n";
 	*w << DisplayKeys(Key.Help)			<< "Help screen\n";
 	*w << DisplayKeys(Key.Playlist)			<< "Playlist screen\n";
 	*w << DisplayKeys(Key.Browser)			<< "Browse screen\n";
@@ -183,6 +190,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.ToggleConsume)		<< "Toggle consume mode\n";
 	if (Mpd.Version() >= 16)
 		*w << DisplayKeys(Key.ToggleReplayGainMode)	<< "Toggle replay gain mode\n";
+	*w << DisplayKeys(Key.ToggleBitrateVisibility)	<< "Toggle bitrate visibility\n";
 	*w << DisplayKeys(Key.Shuffle)			<< "Shuffle playlist\n";
 	*w << DisplayKeys(Key.ToggleCrossfade)		<< "Toggle crossfade mode\n";
 	*w << DisplayKeys(Key.SetCrossfade)		<< "Set crossfade\n";
@@ -195,8 +203,13 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.NextFoundPosition)	<< "Go to next found position\n";
 	*w << DisplayKeys(Key.ToggleFindMode)		<< "Toggle find mode (normal/wrapped)\n";
 	*w << DisplayKeys(Key.GoToContainingDir)	<< "Locate song in browser\n";
+	*w << DisplayKeys(Key.GoToMediaLibrary)		<< "Locate current song in media library\n";
+#	ifdef HAVE_TAGLIB_H
+	*w << DisplayKeys(Key.GoToTagEditor)		<< "Locate current song in tag editor\n";
+#	endif // HAVE_TAGLIB_H
 	*w << DisplayKeys(Key.ToggleDisplayMode)	<< "Toggle display mode\n";
 	*w << DisplayKeys(Key.ToggleInterface)		<< "Toggle user interface\n";
+	*w << DisplayKeys(Key.ToggleSeparatorsInPlaylist) << "Toggle displaying separators between albums in playlist\n";
 	*w << DisplayKeys(Key.GoToPosition)		<< "Go to given position in current song (in % by default)\n";
 	*w << DisplayKeys(Key.SongInfo)			<< "Show song info\n";
 #	ifdef HAVE_CURL_CURL_H
@@ -210,7 +223,7 @@ void Help::GetKeybindings()
 	
 	*w << "\n\n   " << fmtBold << "Keys - Playlist\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Enter)			<< "Play\n";
-	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Add random songs to playlist\n";
+	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Add random songs/artists/albums to playlist\n";
 	*w << DisplayKeys(Key.Delete)			<< "Delete item/selected items from playlist\n";
 	*w << DisplayKeys(Key.Clear)			<< "Clear playlist\n";
 	*w << DisplayKeys(Key.Crop)			<< "Clear playlist but hold currently playing/selected items\n";
@@ -241,6 +254,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.GoToNowPlaying)		<< "Locate currently playing song\n";
 	*w << DisplayKeys(Key.GoToParentDir)		<< "Go to parent directory\n";
 	*w << DisplayKeys(Key.Delete)			<< "Delete playlist/file/directory\n";
+	*w << DisplayKeys(Key.GoToContainingDir)	<< "Jump to playlist editor (playlists only)\n";
 	
 	
 	*w << "\n\n   " << fmtBold << "Keys - Search engine\n -----------------------------------------\n" << fmtBoldEnd;
@@ -250,10 +264,12 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.EditTags)			<< "Edit song's tags\n";
 #	endif // HAVE_TAGLIB_H
 	*w << DisplayKeys(Key.ToggleSingle)		<< "Start searching immediately\n";
+	*w << DisplayKeys(Key.SearchEngine)		<< "Reset search engine\n";
 	
 	
 	*w << "\n\n   " << fmtBold << "Keys - Media library\n -----------------------------------------\n" << fmtBoldEnd;
-	*w << DisplayKeys(Key.MediaLibrary)		<< "Switch between two/three columns\n";
+	if (!Config.media_library_disable_two_column_mode)
+		*w << DisplayKeys(Key.MediaLibrary)	<< "Switch between two/three columns\n";
 	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
 	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
 	*w << DisplayKeys(Key.Enter)			<< "Add to playlist and play song/album/artist's songs\n";
