@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -42,22 +42,7 @@ class CaseInsensitiveStringComparison
 	}
 	
 	public:
-		int operator()(const std::string &a, const std::string &b)
-		{
-			const char *i = a.c_str();
-			const char *j = b.c_str();
-			if (Config.ignore_leading_the)
-			{
-				if (hasTheWord(a))
-					i += 4;
-				if (hasTheWord(b))
-					j += 4;
-			}
-			int dist;
-			while (!(dist = tolower(*i)-tolower(*j)) && *j)
-				++i, ++j;
-			return dist;
-		}
+		int operator()(const std::string &a, const std::string &b);
 };
 
 class CaseInsensitiveSorting
