@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -29,8 +29,9 @@
 #include "playlist.h"
 #include "status.h"
 
-using namespace MPD;
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
+using Global::myScreen;
 
 MediaLibrary *myLibrary = new MediaLibrary;
 
@@ -41,6 +42,12 @@ size_t MediaLibrary::itsMiddleColStartX;
 size_t MediaLibrary::itsRightColWidth;
 size_t MediaLibrary::itsRightColStartX;
 
+// this string marks the position in middle column that works as "All tracks" option. it's
+// assigned to Year in SearchConstraint class since date normally cannot contain other chars
+// than ciphers and -'s (0x7f is interpreted as backspace keycode, so it's quite safe to assume
+// that it won't appear in any tag, let alone date).
+const char MediaLibrary::AllTracksMarker[] = "\x7f";
+
 void MediaLibrary::Init()
 {
 	hasTwoColumns = 0;
@@ -53,21 +60,25 @@ void MediaLibrary::Init()
 	Artists = new Menu<std::string>(0, MainStartY, itsLeftColWidth, MainHeight, IntoStr(Config.media_lib_primary_tag) + "s", Config.main_color, brNone);
 	Artists->HighlightColor(Config.active_column_color);
 	Artists->CyclicScrolling(Config.use_cyclic_scrolling);
+	Artists->CenteredCursor(Config.centered_cursor);
 	Artists->SetSelectPrefix(&Config.selected_item_prefix);
 	Artists->SetSelectSuffix(&Config.selected_item_suffix);
 	Artists->SetItemDisplayer(Display::Generic);
 	
-	Albums = new Menu< std::pair<std::string, SearchConstraints> >(itsMiddleColStartX, MainStartY, itsMiddleColWidth, MainHeight, "Albums", Config.main_color, brNone);
+	Albums = new Menu<SearchConstraints>(itsMiddleColStartX, MainStartY, itsMiddleColWidth, MainHeight, "Albums", Config.main_color, brNone);
 	Albums->HighlightColor(Config.main_highlight_color);
 	Albums->CyclicScrolling(Config.use_cyclic_scrolling);
+	Albums->CenteredCursor(Config.centered_cursor);
 	Albums->SetSelectPrefix(&Config.selected_item_prefix);
 	Albums->SetSelectSuffix(&Config.selected_item_suffix);
-	Albums->SetItemDisplayer(Display::Pairs);
-	Albums->SetGetStringFunction(StringPairToString);
+	Albums->SetItemDisplayer(DisplayAlbums);
+	Albums->SetGetStringFunction(AlbumToString);
+	Albums->SetGetStringFunctionUserData(this);
 	
-	Songs = new Menu<Song>(itsRightColStartX, MainStartY, itsRightColWidth, MainHeight, "Songs", Config.main_color, brNone);
+	Songs = new Menu<MPD::Song>(itsRightColStartX, MainStartY, itsRightColWidth, MainHeight, "Songs", Config.main_color, brNone);
 	Songs->HighlightColor(Config.main_highlight_color);
 	Songs->CyclicScrolling(Config.use_cyclic_scrolling);
+	Songs->CenteredCursor(Config.centered_cursor);
 	Songs->SetSelectPrefix(&Config.selected_item_prefix);
 	Songs->SetSelectSuffix(&Config.selected_item_suffix);
 	Songs->SetItemDisplayer(Display::Songs);
@@ -125,22 +136,27 @@ void MediaLibrary::SwitchTo()
 {
 	if (myScreen == this)
 	{
-		hasTwoColumns = !hasTwoColumns;
-		hasToBeResized = 1;
-		Artists->Clear();
-		Albums->Clear();
-		Albums->Reset();
-		Songs->Clear();
-		if (hasTwoColumns)
-		{
-			if (w == Artists)
-				NextColumn();
-			std::string item_type = IntoStr(Config.media_lib_primary_tag);
-			ToLower(item_type);
-			Albums->SetTitle("Albums (sorted by " + item_type + ")");
-		}
+		if (Config.media_library_disable_two_column_mode)
+			return;
 		else
-			Albums->SetTitle("Albums");
+		{
+			hasTwoColumns = !hasTwoColumns;
+			hasToBeResized = 1;
+			Artists->Clear();
+			Albums->Clear();
+			Albums->Reset();
+			Songs->Clear();
+			if (hasTwoColumns)
+			{
+				if (w == Artists)
+					NextColumn();
+				std::string item_type = IntoStr(Config.media_lib_primary_tag);
+				ToLower(item_type);
+				Albums->SetTitle("Albums (sorted by " + item_type + ")");
+			}
+			else
+				Albums->SetTitle("Albums");
+		}
 	}
 	
 	if (!isInitialized)
@@ -150,9 +166,9 @@ void MediaLibrary::SwitchTo()
 		Resize();
 	
 	if (myScreen != this && myScreen->isTabbable())
-		myPrevScreen = myScreen;
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 	Refresh();
 	UpdateSongList(Songs);
 }
@@ -166,11 +182,12 @@ void MediaLibrary::Update()
 {
 	if (!hasTwoColumns && Artists->Empty())
 	{
-		TagList list;
+		MPD::TagList list;
 		Albums->Clear();
+		Songs->Clear();
 		Mpd.GetList(list, Config.media_lib_primary_tag);
 		sort(list.begin(), list.end(), CaseInsensitiveSorting());
-		for (TagList::iterator it = list.begin(); it != list.end(); ++it)
+		for (MPD::TagList::iterator it = list.begin(); it != list.end(); ++it)
 		{
 			if (!it->empty())
 			{
@@ -182,96 +199,98 @@ void MediaLibrary::Update()
 		Artists->Refresh();
 	}
 	
-	if (!hasTwoColumns && !Artists->Empty() && Albums->Empty())
+	if (!hasTwoColumns && !Artists->Empty() && Albums->Empty() && Songs->Empty())
 	{
-		Songs->Clear();
 		Albums->Reset();
-		TagList list;
+		MPD::TagList list;
 		locale_to_utf(Artists->Current());
-		if (Config.media_lib_primary_tag == MPD_TAG_ARTIST)
-			Mpd.GetAlbums(Artists->Current(), list);
-		else
-		{
-			Mpd.StartFieldSearch(MPD_TAG_ALBUM);
-			Mpd.AddSearch(Config.media_lib_primary_tag, Artists->Current());
-			Mpd.CommitSearch(list);
-		}
+		Mpd.StartFieldSearch(MPD_TAG_ALBUM);
+		Mpd.AddSearch(Config.media_lib_primary_tag, Artists->Current());
+		Mpd.CommitSearch(list);
 		
-		// <mpd-0.14 doesn't support searching for empty tag
-		if (Mpd.Version() > 13)
+		for (MPD::TagList::iterator it = list.begin(); it != list.end(); ++it)
 		{
-			SongList noalbum_list;
-			Mpd.StartSearch(1);
-			Mpd.AddSearch(Config.media_lib_primary_tag, Artists->Current());
-			Mpd.AddSearch(MPD_TAG_ALBUM, "");
-			Mpd.CommitSearch(noalbum_list);
-			if (!noalbum_list.empty())
-				Albums->AddOption(std::make_pair("<no album>", SearchConstraints("", "")));
-			FreeSongList(noalbum_list);
-		}
-		
-		for (TagList::const_iterator it = list.begin(); it != list.end(); ++it)
-		{
-			TagList l;
-			Mpd.StartFieldSearch(MPD_TAG_DATE);
-			Mpd.AddSearch(Config.media_lib_primary_tag, Artists->Current());
-			Mpd.AddSearch(MPD_TAG_ALBUM, *it);
-			Mpd.CommitSearch(l);
-			if (l.empty())
+			if (Config.media_library_display_date)
 			{
-				Albums->AddOption(std::make_pair(*it, SearchConstraints(*it, "")));
-				continue;
+				MPD::TagList l;
+				Mpd.StartFieldSearch(MPD_TAG_DATE);
+				Mpd.AddSearch(Config.media_lib_primary_tag, Artists->Current());
+				Mpd.AddSearch(MPD_TAG_ALBUM, *it);
+				Mpd.CommitSearch(l);
+				utf_to_locale(*it);
+				for (MPD::TagList::iterator j = l.begin(); j != l.end(); ++j)
+				{
+					utf_to_locale(*j);
+					Albums->AddOption(SearchConstraints(*it, *j));
+				}
+			}
+			else
+			{
+				utf_to_locale(*it);
+				Albums->AddOption(SearchConstraints(*it, ""));
 			}
-			for (TagList::const_iterator j = l.begin(); j != l.end(); ++j)
-				Albums->AddOption(std::make_pair("(" + *j + ") " + *it, SearchConstraints(*it, *j)));
 		}
 		utf_to_locale(Artists->Current());
-		for (size_t i = 0; i < Albums->Size(); ++i)
-			utf_to_locale((*Albums)[i].first);
 		if (!Albums->Empty())
-			Albums->Sort<CaseInsensitiveSorting>((*Albums)[0].first == "<no album>");
+			Albums->Sort<SearchConstraintsSorting>();
+		if (Albums->Size() > 1)
+		{
+			Albums->AddSeparator();
+			Albums->AddOption(SearchConstraints("", AllTracksMarker));
+		}
 		Albums->Refresh();
 	}
 	else if (hasTwoColumns && Albums->Empty())
 	{
 		Songs->Clear();
-		TagList artists;
+		MPD::TagList artists;
 		*Albums << XY(0, 0) << "Fetching albums...";
 		Albums->Window::Refresh();
 		Mpd.GetList(artists, Config.media_lib_primary_tag);
-		for (TagList::const_iterator i = artists.begin(); i != artists.end(); ++i)
+		for (MPD::TagList::iterator i = artists.begin(); i != artists.end(); ++i)
 		{
-			TagList albums;
+			if (i->empty())
+				continue;
+			MPD::TagList albums;
 			Mpd.StartFieldSearch(MPD_TAG_ALBUM);
 			Mpd.AddSearch(Config.media_lib_primary_tag, *i);
 			Mpd.CommitSearch(albums);
-			for (TagList::const_iterator j = albums.begin(); j != albums.end(); ++j)
+			for (MPD::TagList::iterator j = albums.begin(); j != albums.end(); ++j)
 			{
-				if (Config.media_lib_primary_tag != MPD_TAG_DATE)
+				if (Config.media_library_display_date)
 				{
-					TagList years;
-					Mpd.StartFieldSearch(MPD_TAG_DATE);
-					Mpd.AddSearch(Config.media_lib_primary_tag, *i);
-					Mpd.AddSearch(MPD_TAG_ALBUM, *j);
-					Mpd.CommitSearch(years);
-					if (!years.empty())
+					if (Config.media_lib_primary_tag != MPD_TAG_DATE)
 					{
-						for (TagList::const_iterator k = years.begin(); k != years.end(); ++k)
+						MPD::TagList years;
+						Mpd.StartFieldSearch(MPD_TAG_DATE);
+						Mpd.AddSearch(Config.media_lib_primary_tag, *i);
+						Mpd.AddSearch(MPD_TAG_ALBUM, *j);
+						Mpd.CommitSearch(years);
+						utf_to_locale(*i);
+						utf_to_locale(*j);
+						for (MPD::TagList::iterator k = years.begin(); k != years.end(); ++k)
 						{
-							Albums->AddOption(std::make_pair(*i + " - (" + *k + ") " + *j, SearchConstraints(*i, *j, *k)));
+							utf_to_locale(*k);
+							Albums->AddOption(SearchConstraints(*i, *j, *k));
 						}
 					}
 					else
-						Albums->AddOption(std::make_pair(*i + " - " + *j, SearchConstraints(*i, *j, "")));
+					{
+						utf_to_locale(*i);
+						utf_to_locale(*j);
+						Albums->AddOption(SearchConstraints(*i, *j, *i));
+					}
 				}
 				else
-					Albums->AddOption(std::make_pair(*i + " - " + *j, SearchConstraints(*i, *j, *i)));
+				{
+					utf_to_locale(*i);
+					utf_to_locale(*j);
+					Albums->AddOption(SearchConstraints(*i, *j, ""));
+				}
 			}
 		}
-		for (size_t i = 0; i < Albums->Size(); ++i)
-			utf_to_locale((*Albums)[i].first);
 		if (!Albums->Empty())
-			Albums->Sort<CaseInsensitiveSorting>();
+			Albums->Sort<SearchConstraintsSorting>();
 		Albums->Refresh();
 	}
 	
@@ -282,13 +301,13 @@ void MediaLibrary::Update()
 		w = Artists;
 	}
 	
-	if ((hasTwoColumns || !Artists->Empty()) && Songs->Empty())
+	if (!(hasTwoColumns ? Albums->Empty() : Artists->Empty()) && Songs->Empty())
 	{
 		Songs->Reset();
-		SongList list;
+		MPD::SongList list;
 		
 		Mpd.StartSearch(1);
-		Mpd.AddSearch(Config.media_lib_primary_tag, hasTwoColumns ? Albums->Current().second.Artist : locale_to_utf_cpy(Artists->Current()));
+		Mpd.AddSearch(Config.media_lib_primary_tag, locale_to_utf_cpy(hasTwoColumns ? Albums->Current().Artist : Artists->Current()));
 		if (Albums->Empty()) // left for compatibility with <mpd-0.14
 		{
 			*Albums << XY(0, 0) << "No albums found.";
@@ -296,16 +315,20 @@ void MediaLibrary::Update()
 		}
 		else
 		{
-			Mpd.AddSearch(MPD_TAG_ALBUM, Albums->Current().second.Album);
-			if (!Albums->Current().second.Album.empty()) // for <no album>
-				Mpd.AddSearch(MPD_TAG_DATE, Albums->Current().second.Year);
+			if (Albums->Current().Year != AllTracksMarker)
+			{
+				Mpd.AddSearch(MPD_TAG_ALBUM, locale_to_utf_cpy(Albums->Current().Album));
+				if (Config.media_library_display_date)
+					Mpd.AddSearch(MPD_TAG_DATE, locale_to_utf_cpy(Albums->Current().Year));
+			}
 		}
 		Mpd.CommitSearch(list);
 		
-		sort(list.begin(), list.end(), SortSongsByTrack);
+		if (!Albums->Empty()) // for compatibility with mpd < 0.14
+			sort(list.begin(), list.end(), Albums->Current().Year == AllTracksMarker ? SortAllTracks : SortSongsByTrack);
 		bool bold = 0;
 		
-		for (SongList::const_iterator it = list.begin(); it != list.end(); ++it)
+		for (MPD::SongList::const_iterator it = list.begin(); it != list.end(); ++it)
 		{
 			for (size_t j = 0; j < myPlaylist->Items->Size(); ++j)
 			{
@@ -329,9 +352,19 @@ void MediaLibrary::SpacePressed()
 	if (Config.space_selects)
 	{
 		if (w == Artists)
+		{
 			Artists->Select(Artists->Choice(), !Artists->isSelected());
+			Albums->Clear();
+			Songs->Clear();
+		}
 		else if (w == Albums)
-			Albums->Select(Albums->Choice(), !Albums->isSelected());
+		{
+			if (Albums->Current().Year != AllTracksMarker)
+			{
+				Albums->Select(Albums->Choice(), !Albums->isSelected());
+				Songs->Clear();
+			}
+		}
 		else if (w == Songs)
 			Songs->Select(Songs->Choice(), !Songs->isSelected());
 		w->Scroll(wDown);
@@ -473,11 +506,10 @@ void MediaLibrary::GetSelectedSongs(MPD::SongList &v)
 				MPD::SongList list;
 				Mpd.StartSearch(1);
 				Mpd.AddSearch(Config.media_lib_primary_tag, hasTwoColumns
-						?  Albums->at(*it).second.Artist
+						?  Albums->at(*it).Artist
 						: locale_to_utf_cpy(Artists->Current()));
-				Mpd.AddSearch(MPD_TAG_ALBUM, Albums->at(*it).second.Album);
-				if (!Albums->at(*it).second.Album.empty()) // for <no album>
-					Mpd.AddSearch(MPD_TAG_DATE, Albums->at(*it).second.Year);
+				Mpd.AddSearch(MPD_TAG_ALBUM, Albums->at(*it).Album);
+				Mpd.AddSearch(MPD_TAG_DATE, Albums->at(*it).Year);
 				Mpd.CommitSearch(list);
 				for (MPD::SongList::const_iterator sIt = list.begin(); sIt != list.end(); ++sIt)
 					v.push_back(new MPD::Song(**sIt));
@@ -542,6 +574,114 @@ void MediaLibrary::PrevColumn()
 	}
 }
 
+void MediaLibrary::LocateSong(const MPD::Song &s)
+{
+	if (Mpd.Version() < 14)
+	{
+		// <mpd-0.14.* has no ability to search for empty tags, which sometimes
+		// leaves albums column empty. since this function relies on this column
+		// being non-empty, it has to be disabled for these versions.
+		ShowMessage("Your MPD version is too old to handle this function properly, please upgrade.");
+		return;
+	}
+	std::string primary_tag;
+	switch (Config.media_lib_primary_tag)
+	{
+		case MPD_TAG_ARTIST:
+			primary_tag = s.GetArtist();
+			break;
+		case MPD_TAG_DATE:
+			primary_tag = s.GetDate();
+			break;
+		case MPD_TAG_GENRE:
+			primary_tag = s.GetGenre();
+			break;
+		case MPD_TAG_COMPOSER:
+			primary_tag = s.GetComposer();
+			break;
+		case MPD_TAG_PERFORMER:
+			primary_tag = s.GetPerformer();
+			break;
+		default:
+			ShowMessage("Invalid tag type in left column of the media library");
+			return;
+	}
+	if (primary_tag.empty())
+	{
+		std::string item_type = IntoStr(Config.media_lib_primary_tag);
+		ToLower(item_type);
+		ShowMessage("Can't jump to media library because the song has no %s tag set.", item_type.c_str());
+		return;
+	}
+	
+	if (myScreen != this)
+		SwitchTo();
+	Statusbar() << "Jumping to song...";
+	Global::wFooter->Refresh();
+	
+	if (!hasTwoColumns)
+	{
+		if (Artists->Empty())
+			Update();
+		if (primary_tag != Artists->Current())
+		{
+			for (size_t i = 0; i < Artists->Size(); ++i)
+			{
+				if (primary_tag == (*Artists)[i])
+				{
+					Artists->Highlight(i);
+					Albums->Clear();
+					Songs->Clear();
+					break;
+				}
+			}
+		}
+	}
+	
+	if (Albums->Empty())
+		Update();
+	
+	std::string album = s.GetAlbum();
+	std::string date = s.GetDate();
+	if ((hasTwoColumns && Albums->Current().Artist != primary_tag)
+	||  album != Albums->Current().Album
+	||  date != Albums->Current().Year)
+	{
+		for (size_t i = 0; i < Albums->Size(); ++i)
+		{
+			if ((!hasTwoColumns || (*Albums)[i].Artist == primary_tag)
+			&&   album == (*Albums)[i].Album
+			&&   date == (*Albums)[i].Year)
+			{
+				Albums->Highlight(i);
+				Songs->Clear();
+				break;
+			}
+		}
+	}
+	
+	if (Songs->Empty())
+		Update();
+	
+	if (s.GetHash() != Songs->Current().GetHash())
+	{
+		for (size_t i = 0; i < Songs->Size(); ++i)
+		{
+			if (s.GetHash()  == (*Songs)[i].GetHash())
+			{
+				Songs->Highlight(i);
+				break;
+			}
+		}
+	}
+	
+	Artists->HighlightColor(Config.main_highlight_color);
+	Albums->HighlightColor(Config.main_highlight_color);
+	Songs->HighlightColor(Config.active_column_color);
+	w = Songs;
+	Refresh();
+}
+
 void MediaLibrary::AddToPlaylist(bool add_n_play)
 {
 	if (w == Songs && !Songs->Empty())
@@ -553,14 +693,15 @@ void MediaLibrary::AddToPlaylist(bool add_n_play)
 
 		if (myPlaylist->Add(list, add_n_play))
 		{
-			if (!Artists->Empty() && w == Artists)
+			if ((!Artists->Empty() && w == Artists)
+			||  (w == Albums && Albums->Current().Year == AllTracksMarker))
 			{
 				std::string tag_type = IntoStr(Config.media_lib_primary_tag);
 				ToLower(tag_type);
 				ShowMessage("Adding songs of %s \"%s\"", tag_type.c_str(), Artists->Current().c_str());
 			}
 			else if (w == Albums)
-				ShowMessage("Adding songs from album \"%s\"", Albums->Current().second.Album.c_str());
+				ShowMessage("Adding songs from album \"%s\"", Albums->Current().Album.c_str());
 		}
 	}
 
@@ -582,12 +723,39 @@ std::string MediaLibrary::SongToString(const MPD::Song &s, void *)
 	return s.toString(Config.song_library_format);
 }
 
-bool MediaLibrary::SortSongsByYear(Song *a, Song *b)
+std::string MediaLibrary::AlbumToString(const SearchConstraints &sc, void *ptr)
 {
-	return a->GetDate() < b->GetDate();
+	if (sc.Year == AllTracksMarker)
+		return "All tracks";
+	std::string result;
+	if (!sc.Artist.empty())
+		(result += sc.Artist) += " - ";
+	if ((!static_cast<MediaLibrary *>(ptr)->hasTwoColumns || Config.media_lib_primary_tag != MPD_TAG_DATE) && !sc.Year.empty())
+		((result += "(") += sc.Year) += ") ";
+	result += sc.Album.empty() ? "<no album>" : sc.Album;
+	return result;
 }
 
-bool MediaLibrary::SortSongsByTrack(Song *a, Song *b)
+void MediaLibrary::DisplayAlbums(const SearchConstraints &sc, void *, Menu<SearchConstraints> *menu)
+{
+	*menu << AlbumToString(sc, 0);
+}
+
+bool MediaLibrary::SearchConstraintsSorting::operator()(const SearchConstraints &a, const SearchConstraints &b) const
+{
+	int result;
+	CaseInsensitiveStringComparison cmp;
+	if (!a.Artist.empty() || b.Artist.empty())
+	{
+		result = cmp(a.Artist, b.Artist);
+		if (result != 0)
+			return result < 0;
+	}
+	result = cmp(a.Year, b.Year);
+	return (result == 0 ? cmp(a.Album, b.Album) : result) < 0;
+}
+
+bool MediaLibrary::SortSongsByTrack(MPD::Song *a, MPD::Song *b)
 {
 	if (a->GetDisc() == b->GetDisc())
 		return StrToInt(a->GetTrack()) < StrToInt(b->GetTrack());
@@ -595,3 +763,13 @@ bool MediaLibrary::SortSongsByTrack(Song *a, Song *b)
 		return StrToInt(a->GetDisc()) < StrToInt(b->GetDisc());
 }
 
+bool MediaLibrary::SortAllTracks(MPD::Song *a, MPD::Song *b)
+{
+	static MPD::Song::GetFunction gets[] = { &MPD::Song::GetDate, &MPD::Song::GetAlbum, &MPD::Song::GetDisc, 0 };
+	CaseInsensitiveStringComparison cmp;
+	for (MPD::Song::GetFunction *get = gets; *get; ++get)
+		if (int ret = cmp(a->GetTags(*get), b->GetTags(*get)))
+			return ret < 0;
+	return a->GetTrack() < b->GetTrack();
+}
+
