@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -39,7 +39,7 @@
 #include "help.h"
 #include "helpers.h"
 #include "media_library.h"
-#include "misc.h"
+#include "sel_items_adder.h"
 #include "server_info.h"
 #include "lyrics.h"
 #include "playlist.h"
@@ -223,12 +223,12 @@ namespace
 int main(int argc, char *argv[])
 {
 	setlocale(LC_ALL, "");
-	
 	CreateConfigDir();
-	DefaultConfiguration(Config);
-	DefaultKeys(Key);
-	ReadConfiguration(Config);
-	ReadKeys(Key);
+	
+	Config.SetDefaults();
+	Key.SetDefaults();
+	Config.Read();
+	Key.Read();
 	
 	if (getenv("MPD_HOST"))
 		Mpd.SetHostname(getenv("MPD_HOST"));
@@ -275,7 +275,8 @@ int main(int argc, char *argv[])
 	wFooter = new Window(0, footer_start_y, COLS, footer_height, "", Config.statusbar_color, brNone);
 	wFooter->SetTimeout(ncmpcpp_window_timeout);
 	wFooter->SetGetStringHelper(StatusbarGetStringHelper);
-	wFooter->AddFDCallback(Mpd.GetFD(), StatusbarMPDCallback);
+	if (Mpd.SupportsIdle())
+		wFooter->AddFDCallback(Mpd.GetFD(), StatusbarMPDCallback);
 	wFooter->CreateHistory();
 	
 	// initialize screens to browser as default previous screen
@@ -287,6 +288,10 @@ int main(int argc, char *argv[])
 	myPlaylist->SwitchTo();
 	myPlaylist->UpdateTimer();
 	
+	// go to startup screen
+	if (Config.startup_screen != myScreen)
+		Config.startup_screen->SwitchTo();
+	
 	Mpd.SetStatusUpdater(NcmpcppStatusChanged, 0);
 	Mpd.SetErrorHandler(NcmpcppErrorCallback, 0);
 	
@@ -311,6 +316,7 @@ int main(int argc, char *argv[])
 	if (Config.mouse_support)
 		mousemask(ALL_MOUSE_EVENTS, 0);
 	
+	Mpd.OrderDataFetching();
 	if (Config.jump_to_now_playing_song_at_start)
 	{
 		TraceMpdStatus();
@@ -328,7 +334,14 @@ int main(int argc, char *argv[])
 			if (Mpd.Connect())
 			{
 				ShowMessage("Connected to %s!", Mpd.GetHostname().c_str());
-				wFooter->AddFDCallback(Mpd.GetFD(), StatusbarMPDCallback);
+				if (Mpd.SupportsIdle())
+				{
+					wFooter->AddFDCallback(Mpd.GetFD(), StatusbarMPDCallback);
+					Mpd.OrderDataFetching(); // we need info about new connection
+				}
+				// we want everything updated after reconnect, no blocking.
+				Playlist::BlockUpdate = 0;
+				Global::BlockItemListUpdate = 0;
 				MessagesAllowed = 0;
 				UpdateStatusImmediately = 1;
 #				ifdef ENABLE_VISUALIZER
@@ -434,6 +447,51 @@ int main(int argc, char *argv[])
 		{
 			myScreen->Scroll(wDown, Key.Down);
 		}
+		else if (Keypressed(input, Key.UpAlbum)  || Keypressed(input, Key.DownAlbum)
+		     ||  Keypressed(input, Key.UpArtist) || Keypressed(input, Key.DownArtist))
+		{
+			Menu<MPD::Song> *songs = NULL;
+			if (myScreen == myPlaylist && !myPlaylist->Items->Empty())
+				songs = myPlaylist->Items;
+			else if (myScreen->ActiveWindow() == myPlaylistEditor->Content)
+				songs = myPlaylistEditor->Content;
+			else if (myScreen == mySearcher)
+				mySearcher->Scroll(input);
+
+			if (songs && !songs->Empty())
+			{
+				size_t pos = songs->Choice();
+				if (Keypressed(input, Key.UpAlbum))
+				{
+					std::string album = songs->at(pos).GetAlbum();
+					while (pos > 0)
+						if (songs->at(--pos).GetAlbum() != album)
+							break;
+				}
+				else if (Keypressed(input, Key.DownAlbum))
+				{
+					std::string album = songs->at(pos).GetAlbum();
+					while (pos < songs->Size() - 1)
+						if (songs->at(++pos).GetAlbum() != album)
+							break;
+				}
+				else if (Keypressed(input, Key.UpArtist))
+				{
+					std::string artist = songs->at(pos).GetArtist();
+					while (pos > 0)
+						if (songs->at(--pos).GetArtist() != artist)
+							break;
+				}
+				else if (Keypressed(input, Key.DownArtist))
+				{
+					std::string artist = songs->at(pos).GetArtist();
+					while (pos < songs->Size() - 1)
+						if (songs->at(++pos).GetArtist() != artist)
+							break;
+				}
+				songs->Highlight(pos);
+			}
+		}
 		else if (Keypressed(input, Key.PageUp))
 		{
 			myScreen->Scroll(wPageUp, Key.PageUp);
@@ -484,7 +542,7 @@ int main(int argc, char *argv[])
 			else if (mouse_event.bstate & (BUTTON1_PRESSED | BUTTON2_PRESSED | BUTTON3_PRESSED | BUTTON4_PRESSED))
 				myScreen->MouseButtonPressed(mouse_event);
 		}
-		if (Keypressed(input, Key.ToggleInterface))
+		else if (Keypressed(input, Key.ToggleInterface))
 		{
 			Config.new_design = !Config.new_design;
 			Config.statusbar_visibility = Config.new_design ? 0 : real_statusbar_visibility;
@@ -576,9 +634,8 @@ int main(int argc, char *argv[])
 				}
 				else
 				{
-					Playlist::BlockNowPlayingUpdate = 1;
 					wFooter->SetTimeout(50);
-					int del_counter = 0;
+					bool adjust_np = myPlaylist->NowPlaying > int(myPlaylist->CurrentSong()->GetPosition());
 					while (!myPlaylist->Items->Empty() && Keypressed(input, Key.Delete))
 					{
 						size_t id = myPlaylist->Items->Choice();
@@ -586,21 +643,19 @@ int main(int argc, char *argv[])
 						Playlist::BlockUpdate = 1;
 						myPlaylist->UpdateTimer();
 						// needed for keeping proper position of now playing song.
-						if (myPlaylist->NowPlaying > int(myPlaylist->CurrentSong()->GetPosition())-del_counter)
+						if (adjust_np)
 							--myPlaylist->NowPlaying;
 						if (Mpd.DeleteID(myPlaylist->CurrentSong()->GetID()))
 						{
 							myPlaylist->Items->DeleteOption(id);
 							myPlaylist->Items->Refresh();
 							wFooter->ReadKey(input);
-							++del_counter;
 						}
 						else
 							break;
 					}
 					myPlaylist->FixPositions(myPlaylist->Items->Choice());
 					wFooter->SetTimeout(ncmpcpp_window_timeout);
-					Playlist::BlockNowPlayingUpdate = 0;
 				}
 			}
 			else if (
@@ -885,8 +940,8 @@ int main(int argc, char *argv[])
 						if (modify_now_playing)
 							--myPlaylist->NowPlaying;
 						--to;
-						myPlaylist->Items->at(from).SetPosition(to);
-						myPlaylist->Items->at(to).SetPosition(from);
+						myPlaylist->Items->at(to+1).SetPosition(to);
+						myPlaylist->Items->at(to).SetPosition(to+1);
 						myPlaylist->Items->Swap(to, to+1);
 						myPlaylist->Items->Scroll(wUp);
 						myPlaylist->Items->Refresh();
@@ -1011,8 +1066,8 @@ int main(int argc, char *argv[])
 						if (modify_now_playing)
 							++myPlaylist->NowPlaying;
 						++to;
-						myPlaylist->Items->at(from).SetPosition(to);
-						myPlaylist->Items->at(to).SetPosition(from);
+						myPlaylist->Items->at(to-1).SetPosition(to);
+						myPlaylist->Items->at(to).SetPosition(to-1);
 						myPlaylist->Items->Swap(to, to-1);
 						myPlaylist->Items->Scroll(wDown);
 						myPlaylist->Items->Refresh();
@@ -1260,6 +1315,11 @@ int main(int argc, char *argv[])
 					mySearcher->Main()->SetTitle(Config.columns_in_search_engine ? Display::Columns() : "");
 			}
 		}
+		else if (Keypressed(input, Key.ToggleSeparatorsInPlaylist))
+		{
+			Config.playlist_separate_albums = !Config.playlist_separate_albums;
+			ShowMessage("Separators between albums in playlist: %s", Config.playlist_separate_albums ? "On" : "Off");
+		}
 #		ifdef HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.ToggleLyricsDB))
 		{
@@ -1411,7 +1471,7 @@ int main(int argc, char *argv[])
 					if (success)
 					{
 						Mpd.UpdateDirectory(locale_to_utf_cpy(FindSharedDir(list)));
-						ShowMessage("Tags updated succesfully!");
+						ShowMessage("Tags updated successfully!");
 					}
 					FreeSongList(list);
 				}
@@ -1420,9 +1480,9 @@ int main(int argc, char *argv[])
 			{
 				LockStatusbar();
 				Statusbar() << fmtBold << "Album: " << fmtBoldEnd;
-				std::string new_album = wFooter->GetString(myLibrary->Albums->Current().second.Album);
+				std::string new_album = wFooter->GetString(myLibrary->Albums->Current().Album);
 				UnlockStatusbar();
-				if (!new_album.empty() && new_album != myLibrary->Albums->Current().second.Album)
+				if (!new_album.empty() && new_album != myLibrary->Albums->Current().Album)
 				{
 					bool success = 1;
 					ShowMessage("Updating tags...");
@@ -1451,7 +1511,7 @@ int main(int argc, char *argv[])
 					if (success)
 					{
 						Mpd.UpdateDirectory(locale_to_utf_cpy(FindSharedDir(myLibrary->Songs)));
-						ShowMessage("Tags updated succesfully!");
+						ShowMessage("Tags updated successfully!");
 					}
 				}
 			}
@@ -1540,11 +1600,33 @@ int main(int argc, char *argv[])
 			}
 		}
 		else if (Keypressed(input, Key.GoToContainingDir))
+		{
+			if (myScreen == myBrowser && myBrowser->Main()->Current().type == itPlaylist)
+			{
+				const std::string pl_name = myBrowser->Main()->Current().name;
+				myPlaylistEditor->JumpTo(pl_name);
+			}
+			else
+			{
+				Song *s = myScreen->CurrentSong();
+				if (s)
+					myBrowser->LocateSong(*s);
+			}
+		}
+		else if (Keypressed(input, Key.GoToMediaLibrary))
 		{
 			Song *s = myScreen->CurrentSong();
 			if (s)
-				myBrowser->LocateSong(*s);
+				myLibrary->LocateSong(*s);
 		}
+#		ifdef HAVE_TAGLIB_H
+		else if (Keypressed(input, Key.GoToTagEditor))
+		{
+			CHECK_MPD_MUSIC_DIR;
+			if (Song *s = myScreen->CurrentSong())
+				myTagEditor->LocateSong(*s);
+		}
+#		endif // HAVE_TAGLIB_H
 		else if (Keypressed(input, Key.GoToPosition))
 		{
 			if (!Mpd.GetTotalTime())
@@ -1833,16 +1915,37 @@ int main(int argc, char *argv[])
 			mousemask(Config.mouse_support ? ALL_MOUSE_EVENTS : 0, 0);
 			ShowMessage("Mouse support %s", Config.mouse_support ? "enabled" : "disabled");
 		}
+		else if (Keypressed(input, Key.ToggleBitrateVisibility))
+		{
+			Config.display_bitrate = !Config.display_bitrate;
+			ShowMessage("Bitrate visibility %s", Config.display_bitrate ? "enabled" : "disabled");
+		}
 		else if (Keypressed(input, Key.SwitchTagTypeList))
 		{
 			if (myScreen == myPlaylist)
 			{
 				LockStatusbar();
-				Statusbar() << "Number of random songs: ";
+				Statusbar() << "Add random ? [" << fmtBold << 's' << fmtBoldEnd << "ongs/" << fmtBold << 'a' << fmtBoldEnd << "rtists/al" << fmtBold << 'b' << fmtBoldEnd << "ums] ";
+				wFooter->Refresh();
+				int answer = 0;
+				do
+				{
+					TraceMpdStatus();
+					wFooter->ReadKey(answer);
+				}
+				while (answer != 's' && answer != 'a' && answer != 'b');
+				UnlockStatusbar();
+				
+				mpd_tag_type tag_type = IntoTagItem(answer);
+				std::string tag_type_str = answer == 's' ? "song" : IntoStr(tag_type);
+				ToLower(tag_type_str);
+				
+				LockStatusbar();
+				Statusbar() << "Number of random " << tag_type_str << "s: ";
 				size_t number = StrToLong(wFooter->GetString());
 				UnlockStatusbar();
-				if (number && Mpd.AddRandomSongs(number))
-					ShowMessage("%zu random song%s added to playlist!", number, number == 1 ? "" : "s");
+				if (number && (answer == 's' ? Mpd.AddRandomSongs(number) : Mpd.AddRandomTag(tag_type, number)))
+					ShowMessage("%zu random %s%s added to playlist!", number, tag_type_str.c_str(), number == 1 ? "" : "s");
 			}
 			else if (myScreen == myBrowser && !myBrowser->isLocal())
 			{
@@ -1854,7 +1957,7 @@ int main(int argc, char *argv[])
 			||	 (myLibrary->Columns() == 2 && myScreen->ActiveWindow() == myLibrary->Albums))
 			{
 				LockStatusbar();
-				Statusbar() << "Tag type ? [" << fmtBold << 'a' << fmtBoldEnd << "rtist/" << fmtBold << 'y' << fmtBoldEnd << "ear/" << fmtBold << 'g' << fmtBoldEnd << "enre/" << fmtBold << 'c' << fmtBoldEnd << "omposer/" << fmtBold << 'p' << fmtBoldEnd << "erformer] ";
+				Statusbar() << "Tag type ? [" << fmtBold << 'a' << fmtBoldEnd << "rtist/album" << fmtBold << 'A' << fmtBoldEnd << "rtist/" << fmtBold << 'y' << fmtBoldEnd << "ear/" << fmtBold << 'g' << fmtBoldEnd << "enre/" << fmtBold << 'c' << fmtBoldEnd << "omposer/" << fmtBold << 'p' << fmtBoldEnd << "erformer] ";
 				wFooter->Refresh();
 				int answer = 0;
 				do
@@ -1862,7 +1965,7 @@ int main(int argc, char *argv[])
 					TraceMpdStatus();
 					wFooter->ReadKey(answer);
 				}
-				while (answer != 'a' && answer != 'y' && answer != 'g' && answer != 'c' && answer != 'p');
+				while (answer != 'a' && answer != 'A' && answer != 'y' && answer != 'g' && answer != 'c' && answer != 'p');
 				UnlockStatusbar();
 				mpd_tag_type new_tagitem = IntoTagItem(answer);
 				if (new_tagitem != Config.media_lib_primary_tag)
@@ -1923,20 +2026,21 @@ int main(int argc, char *argv[])
 		}
 		else if (Keypressed(input, Key.ScreenSwitcher))
 		{
-			if (Config.screen_switcher_browser_only)
-			{
-				if (myScreen == myPlaylist)
-					myBrowser->SwitchTo();
-				else
-					myPlaylist->SwitchTo();
-			}
-			else
+			if (Config.screen_switcher_previous)
 			{
 				if (myScreen->isTabbable())
 					myPrevScreen->SwitchTo();
 				else
 					myOldScreen->SwitchTo();
 			}
+			else if (!Config.screens_seq.empty())
+			{
+				std::list<BasicScreen *>::const_iterator screen = std::find(Config.screens_seq.begin(), Config.screens_seq.end(), myScreen);
+				if (++screen == Config.screens_seq.end())
+					(*Config.screens_seq.begin())->SwitchTo();
+				else
+					(*screen)->SwitchTo();
+			}
 		}
 		else if (Keypressed(input, Key.Playlist))
 		{
@@ -2011,6 +2115,7 @@ int main(int argc, char *argv[])
 				if (myScreen->ActiveWindow() == myLibrary->Artists)
 				{
 					myLibrary->Albums->Clear();
+					myLibrary->Songs->Clear();
 				}
 				else if (myScreen->ActiveWindow() == myLibrary->Albums)
 				{
