@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -28,7 +28,8 @@
 #include "song.h"
 #include "status.h"
 
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
 
 Playlist *myPlaylist = new Playlist;
 
@@ -48,6 +49,7 @@ void Playlist::Init()
 {
 	Items = new Menu<MPD::Song>(0, MainStartY, COLS, MainHeight, Config.columns_in_playlist ? Display::Columns() : "", Config.main_color, brNone);
 	Items->CyclicScrolling(Config.use_cyclic_scrolling);
+	Items->CenteredCursor(Config.centered_cursor);
 	Items->HighlightColor(Config.main_highlight_color);
 	Items->SetSelectPrefix(&Config.selected_item_prefix);
 	Items->SetSelectSuffix(&Config.selected_item_suffix);
@@ -62,6 +64,7 @@ void Playlist::Init()
 		
 		SortDialog = new Menu< std::pair<std::string, MPD::Song::GetFunction> >((COLS-SortDialogWidth)/2, (MainHeight-SortDialogHeight)/2+MainStartY, SortDialogWidth, SortDialogHeight, "Sort songs by...", Config.main_color, Config.window_border);
 		SortDialog->CyclicScrolling(Config.use_cyclic_scrolling);
+		SortDialog->CenteredCursor(Config.centered_cursor);
 		SortDialog->SetItemDisplayer(Display::Pairs);
 		
 		SortDialog->AddOption(std::make_pair("Artist", &MPD::Song::GetArtist));
@@ -86,6 +89,8 @@ void Playlist::Init()
 
 void Playlist::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -98,13 +103,13 @@ void Playlist::SwitchTo()
 		Resize();
 	
 	if (myScreen != this && myScreen->isTabbable())
-		myPrevScreen = myScreen;
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	Items->Window::Clear();
 	EnableHighlighting();
 	if (w != Items) // even if sorting window is active, background has to be refreshed anyway
 		Items->Display();
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 }
 
 void Playlist::Resize()
@@ -132,7 +137,7 @@ std::basic_string<my_char_t> Playlist::Title()
 	std::basic_string<my_char_t> result = U("Playlist ");
 	if (ReloadTotalLength || ReloadRemaining)
 		itsBufferedStats = TotalLength();
-	result += Scroller(TO_WSTRING(itsBufferedStats), itsScrollBegin, Items->GetWidth()-result.length()-(Config.new_design ? 2 : VolumeState.length()));
+	result += Scroller(TO_WSTRING(itsBufferedStats), itsScrollBegin, Items->GetWidth()-result.length()-(Config.new_design ? 2 : Global::VolumeState.length()));
 	return result;
 }
 
@@ -143,7 +148,7 @@ void Playlist::EnterPressed()
 		if (!Items->Empty())
 		{
 			Mpd.PlayID(Items->Current().GetID());
-			UpdateStatusImmediately = 1;
+			Global::UpdateStatusImmediately = 1;
 		}
 	}
 	else if (w == SortDialog)
@@ -236,6 +241,12 @@ void Playlist::SpacePressed()
 	}
 }
 
+void Playlist::ReadKey(int &key)
+{
+	w->ReadKey(key);
+	UpdateTimer();
+}
+
 void Playlist::MouseButtonPressed(MEVENT me)
 {
 	if (w == Items && !Items->Empty() && Items->hasCoords(me.x, me.y))
@@ -269,6 +280,8 @@ MPD::Song *Playlist::CurrentSong()
 
 void Playlist::GetSelectedSongs(MPD::SongList &v)
 {
+	if (myPlaylist->Items->Empty())
+		return;
 	std::vector<size_t> selected;
 	Items->GetSelected(selected);
 	if (selected.empty())
@@ -374,12 +387,12 @@ std::string Playlist::TotalLength()
 	if (itsTotalLength)
 	{
 		result << ", length: ";
-		ShowTime(result, itsTotalLength, 0);
+		ShowTime(result, itsTotalLength, Config.playlist_shorten_total_times);
 	}
 	if (Config.playlist_show_remaining_time && itsRemainingTime && !Items->isFiltered() && Items->Size() > 1)
 	{
 		result << " :: remaining: ";
-		ShowTime(result, itsRemainingTime, 0);
+		ShowTime(result, itsRemainingTime, Config.playlist_shorten_total_times);
 	}
 	result << ')';
 	return result.str();
@@ -402,30 +415,12 @@ std::string Playlist::SongToString(const MPD::Song &s, void *data)
 
 std::string Playlist::SongInColumnsToString(const MPD::Song &s, void *)
 {
-	std::string result = "{";
-	for (std::vector<Column>::const_iterator it = Config.columns.begin(); it != Config.columns.end(); ++it)
-	{
-		if (it->type == 't')
-		{
-			result += "{%t}|{%f}";
-		}
-		else
-		{
-			// tags should be put in additional braces as if they are not, the
-			// tag that is not present within 'main' braces discards them all.
-			result += "{%";
-			result += it->type;
-			result += "}";
-		}
-		result += " ";
-	}
-	result += "}";
-	return s.toString(result);
+	return s.toString(Config.song_in_columns_to_string_format);
 }
 
 bool Playlist::Add(const MPD::Song &s, bool in_playlist, bool play, int position)
 {
-	BlockItemListUpdate = 1;
+	Global::BlockItemListUpdate = 1;
 	if (Config.ncmpc_like_songs_adding && in_playlist)
 	{
 		unsigned hash = s.GetHash();
