@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,7 +21,6 @@
 #ifndef _SETTINGS_H
 #define _SETTINGS_H
 
-#include <limits>
 #include <vector>
 
 #include <mpd/client.h>
@@ -36,24 +35,35 @@
 #endif // WIN32
 
 const std::string config_dir = home_path + HOME_FOLDER;
-const int null_key = std::numeric_limits<int>::max();
+
+class BasicScreen; // forward declaration for screens sequence
 
 struct Column
 {
 	Column() : right_alignment(0), display_empty_tag(1) { }
 	
+	std::basic_string<my_char_t> name;
+	std::string type;
 	unsigned width;
 	Color color;
-	char type;
 	bool fixed;
 	bool right_alignment;
 	bool display_empty_tag;
 };
 
-struct ncmpcpp_keys
+struct NcmpcppKeys
 {
+	static const int NullKey = 0;
+	
+	void SetDefaults();
+	void Read();
+	
 	int Up[2];
 	int Down[2];
+	int UpAlbum[2];
+	int DownAlbum[2];
+	int UpArtist[2];
+	int DownArtist[2];
 	int PageUp[2];
 	int PageDown[2];
 	int Home[2];
@@ -89,6 +99,7 @@ struct ncmpcpp_keys
 	int ToggleSpaceMode[2];
 	int ToggleAddMode[2];
 	int ToggleMouse[2];
+	int ToggleBitrateVisibility[2];
 	int Shuffle[2];
 	int ToggleCrossfade[2];
 	int SetCrossfade[2];
@@ -117,17 +128,23 @@ struct ncmpcpp_keys
 	int SavePlaylist[2];
 	int GoToNowPlaying[2];
 	int GoToContainingDir[2];
+	int GoToMediaLibrary[2];
+	int GoToTagEditor[2];
 	int ToggleAutoCenter[2];
 	int ToggleDisplayMode[2];
 	int ToggleInterface[2];
+	int ToggleSeparatorsInPlaylist[2];
 	int ToggleLyricsDB[2];
 	int GoToParentDir[2];
 	int SwitchTagTypeList[2];
 	int Quit[2];
 };
 
-struct ncmpcpp_config
+struct NcmpcppConfig
 {
+	void SetDefaults();
+	void Read();
+	
 	std::string mpd_host;
 	std::string mpd_music_dir;
 	std::string visualizer_fifo_path;
@@ -140,6 +157,7 @@ struct ncmpcpp_config
 	std::string song_window_title_format;
 	std::string song_library_format;
 	std::string tag_editor_album_format;
+	std::string song_in_columns_to_string_format;
 	std::string external_editor;
 	std::string system_encoding;
 	std::string execute_on_song_change;
@@ -180,6 +198,8 @@ struct ncmpcpp_config
 	bool colors_enabled;
 	bool fancy_scrolling;
 	bool playlist_show_remaining_time;
+	bool playlist_shorten_total_times;
+	bool playlist_separate_albums;
 	bool columns_in_playlist;
 	bool columns_in_browser;
 	bool columns_in_search_engine;
@@ -188,7 +208,7 @@ struct ncmpcpp_config
 	bool header_text_scrolling;
 	bool statusbar_visibility;
 	bool centered_cursor;
-	bool screen_switcher_browser_only;
+	bool screen_switcher_previous;
 	bool autocenter_mode;
 	bool wrapped_search;
 	bool space_selects;
@@ -201,6 +221,7 @@ struct ncmpcpp_config
 	bool display_screens_numbers_on_start;
 	bool jump_to_now_playing_song_at_start;
 	bool clock_display_seconds;
+	bool display_volume_level;
 	bool display_bitrate;
 	bool display_remaining_time;
 	bool ignore_leading_the;
@@ -216,6 +237,9 @@ struct ncmpcpp_config
 	bool visualizer_use_wave;
 	bool browser_sort_by_mtime;
 	bool tag_editor_extended_numeration;
+	bool media_library_display_date;
+	bool media_library_disable_two_column_mode;
+	bool discard_colors_if_item_is_selected;
 	
 	int mpd_port;
 	int mpd_connection_timeout;
@@ -227,19 +251,23 @@ struct ncmpcpp_config
 	int regex_type;
 	
 	unsigned lines_scrolled;
+	unsigned search_engine_default_search_mode;
+	unsigned visualizer_sync_interval;
 	
 	size_t selected_item_suffix_length;
+	size_t now_playing_suffix_length;
+	
+	BasicScreen *startup_screen;
+	std::list<BasicScreen *> screens_seq;
 };
 
-extern ncmpcpp_config Config;
-extern ncmpcpp_keys Key;
+extern NcmpcppKeys Key;
+extern NcmpcppConfig Config;
 
 void CreateConfigDir();
 void SetWindowsDimensions(size_t &header_height, size_t &footer_start_y, size_t &footer_height);
-void DefaultKeys(ncmpcpp_keys &);
-void DefaultConfiguration(ncmpcpp_config &);
-void ReadKeys(ncmpcpp_keys &);
-void ReadConfiguration(ncmpcpp_config &);
+void ReadKeys(NcmpcppKeys &);
+void ReadConfiguration(NcmpcppConfig &);
 
 #endif
 
