@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -190,6 +190,11 @@ std::string MPD::Song::GetAlbum(unsigned pos) const
 	return GetTag(MPD_TAG_ALBUM, pos);
 }
 
+std::string MPD::Song::GetAlbumArtist(unsigned pos) const
+{
+	return GetTag(MPD_TAG_ALBUM_ARTIST, pos);
+}
+
 std::string MPD::Song::GetTrack(unsigned pos) const
 {
 	std::string track = GetTag(MPD_TAG_TRACK, pos);
@@ -263,6 +268,11 @@ void MPD::Song::SetAlbum(const std::string &str, unsigned pos)
 	SetTag(MPD_TAG_ALBUM, pos, str);
 }
 
+void MPD::Song::SetAlbumArtist(const std::string &str, unsigned pos)
+{
+	SetTag(MPD_TAG_ALBUM_ARTIST, pos, str);
+}
+
 void MPD::Song::SetTrack(const std::string &str, unsigned pos)
 {
 	SetTag(MPD_TAG_TRACK, pos, str);
@@ -374,6 +384,9 @@ std::string MPD::Song::ParseFormat(std::string::const_iterator &it, const char *
 				case 'a':
 					get = &MPD::Song::GetArtist;
 					break;
+				case 'A':
+					get = &MPD::Song::GetAlbumArtist;
+					break;
 				case 'b':
 					get = &MPD::Song::GetAlbum;
 					break;
@@ -416,7 +429,7 @@ std::string MPD::Song::ParseFormat(std::string::const_iterator &it, const char *
 				if (escape_chars) // prepend format escape character to all given chars to escape
 					for (const char *ch = escape_chars; *ch; ++ch)
 						for (size_t i = 0; (i = tag.find(*ch, i)) != std::string::npos; i += 2)
-							tag.replace(i, 1, std::string(1, FormatEscapeCharacter) + ch);
+							tag.replace(i, 1, std::string(1, FormatEscapeCharacter) + *ch);
 				if (!tag.empty() && (get != &MPD::Song::GetLength || GetTotalLength()))
 				{
 					has_some_tags = 1;
