@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,14 +36,11 @@
 #include "charset.h"
 #include "display.h"
 #include "global.h"
+#include "info.h"
 #include "playlist.h"
 
 using Global::MainHeight;
 using Global::MainStartY;
-using Global::myOldScreen;
-using Global::myScreen;
-using Global::myPrevScreen;
-using Global::wFooter;
 
 TagEditor *myTagEditor = new TagEditor;
 
@@ -70,12 +67,14 @@ void TagEditor::Init()
 	Albums = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Albums", Config.main_color, brNone);
 	Albums->HighlightColor(Config.active_column_color);
 	Albums->CyclicScrolling(Config.use_cyclic_scrolling);
+	Albums->CenteredCursor(Config.centered_cursor);
 	Albums->SetItemDisplayer(Display::Pairs);
 	Albums->SetGetStringFunction(StringPairToString);
 	
 	Dirs = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Directories", Config.main_color, brNone);
 	Dirs->HighlightColor(Config.active_column_color);
 	Dirs->CyclicScrolling(Config.use_cyclic_scrolling);
+	Dirs->CenteredCursor(Config.centered_cursor);
 	Dirs->SetItemDisplayer(Display::Pairs);
 	Dirs->SetGetStringFunction(StringPairToString);
 	
@@ -84,18 +83,11 @@ void TagEditor::Init()
 	TagTypes = new Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, "Tag types", Config.main_color, brNone);
 	TagTypes->HighlightColor(Config.main_highlight_color);
 	TagTypes->CyclicScrolling(Config.use_cyclic_scrolling);
+	TagTypes->CenteredCursor(Config.centered_cursor);
 	TagTypes->SetItemDisplayer(Display::Generic);
 	
-	TagTypes->AddOption("Title");
-	TagTypes->AddOption("Artist");
-	TagTypes->AddOption("Album");
-	TagTypes->AddOption("Year");
-	TagTypes->AddOption("Track");
-	TagTypes->AddOption("Genre");
-	TagTypes->AddOption("Composer");
-	TagTypes->AddOption("Performer");
-	TagTypes->AddOption("Disc");
-	TagTypes->AddOption("Comment");
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+		TagTypes->AddOption(m->Name);
 	TagTypes->AddSeparator();
 	TagTypes->AddOption("Filename");
 	TagTypes->AddSeparator();
@@ -110,6 +102,7 @@ void TagEditor::Init()
 	Tags = new Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, "Tags", Config.main_color, brNone);
 	Tags->HighlightColor(Config.main_highlight_color);
 	Tags->CyclicScrolling(Config.use_cyclic_scrolling);
+	Tags->CenteredCursor(Config.centered_cursor);
 	Tags->SetSelectPrefix(&Config.selected_item_prefix);
 	Tags->SetSelectSuffix(&Config.selected_item_suffix);
 	Tags->SetItemDisplayer(Display::Tags);
@@ -119,6 +112,7 @@ void TagEditor::Init()
 	
 	FParserDialog = new Menu<std::string>((COLS-FParserDialogWidth)/2, (MainHeight-FParserDialogHeight)/2+MainStartY, FParserDialogWidth, FParserDialogHeight, "", Config.main_color, Config.window_border);
 	FParserDialog->CyclicScrolling(Config.use_cyclic_scrolling);
+	FParserDialog->CenteredCursor(Config.centered_cursor);
 	FParserDialog->SetItemDisplayer(Display::Generic);
 	FParserDialog->AddOption("Get tags from filename");
 	FParserDialog->AddOption("Rename files");
@@ -127,6 +121,7 @@ void TagEditor::Init()
 	
 	FParser = new Menu<std::string>((COLS-FParserWidth)/2, (MainHeight-FParserHeight)/2+MainStartY, FParserWidthOne, FParserHeight, "_", Config.main_color, Config.active_window_border);
 	FParser->CyclicScrolling(Config.use_cyclic_scrolling);
+	FParser->CenteredCursor(Config.centered_cursor);
 	FParser->SetItemDisplayer(Display::Generic);
 	
 	FParserLegend = new Scrollpad((COLS-FParserWidth)/2+FParserWidthOne, (MainHeight-FParserHeight)/2+MainStartY, FParserWidthTwo, FParserHeight, "Legend", Config.main_color, Config.window_border);
@@ -189,6 +184,8 @@ std::basic_string<my_char_t> TagEditor::Title()
 
 void TagEditor::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -199,7 +196,7 @@ void TagEditor::SwitchTo()
 		Resize();
 	
 	if (myScreen != this && myScreen->isTabbable())
-		myPrevScreen = myScreen;
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	Global::RedrawHeader = 1;
 	Refresh();
@@ -235,7 +232,7 @@ void TagEditor::Update()
 		{
 			*Albums << XY(0, 0) << "Fetching albums...";
 			Albums->Window::Refresh();
-			Mpd.GetAlbums("", list);
+			Mpd.GetList(list, MPD_TAG_ALBUM);
 			for (MPD::TagList::const_iterator it = list.begin(); it != list.end(); ++it)
 			{
 				MPD::SongList l;
@@ -288,14 +285,17 @@ void TagEditor::Update()
 		MPD::SongList list;
 		if (Config.albums_in_tag_editor)
 		{
-			Mpd.StartSearch(1);
-			Mpd.AddSearch(MPD_TAG_ALBUM, Albums->Current().second);
-			Mpd.CommitSearch(list);
-			sort(list.begin(), list.end(), CaseInsensitiveSorting());
-			for (MPD::SongList::iterator it = list.begin(); it != list.end(); ++it)
+			if (!Albums->Empty())
 			{
-				(*it)->Localize();
-				Tags->AddOption(**it);
+				Mpd.StartSearch(1);
+				Mpd.AddSearch(MPD_TAG_ALBUM, Albums->Current().second);
+				Mpd.CommitSearch(list);
+				sort(list.begin(), list.end(), CaseInsensitiveSorting());
+				for (MPD::SongList::iterator it = list.begin(); it != list.end(); ++it)
+				{
+					(*it)->Localize();
+					Tags->AddOption(**it);
+				}
 			}
 		}
 		else
@@ -318,11 +318,16 @@ void TagEditor::Update()
 		Tags->Refresh();
 	}
 	else if (TagTypes->Choice() >= 13)
+	{
 		Tags->Window::Clear();
+		Tags->Window::Refresh();
+	}
 }
 
 void TagEditor::EnterPressed()
 {
+	using Global::wFooter;
+	
 	if (w == Dirs)
 	{
 		MPD::TagList test;
@@ -352,6 +357,7 @@ void TagEditor::EnterPressed()
 		
 		FParserLegend->Clear();
 		*FParserLegend << "%a - artist\n";
+		*FParserLegend << "%A - album artist\n";
 		*FParserLegend << "%t - title\n";
 		*FParserLegend << "%b - album\n";
 		*FParserLegend << "%y - year\n";
@@ -508,81 +514,65 @@ void TagEditor::EnterPressed()
 		for (size_t i = 0; i < Tags->Size(); ++i)
 			EditedSongs.push_back(&(*Tags)[i]);
 	
-	MPD::Song::GetFunction get = 0;
-	MPD::Song::SetFunction set = 0;
-	
 	size_t id = TagTypes->RealChoice();
-	switch (id)
+	
+	if (w == TagTypes && id == 5)
 	{
-		case 0:
-			get = &MPD::Song::GetTitle;
-			set = &MPD::Song::SetTitle;
-			break;
-		case 1:
-			get = &MPD::Song::GetArtist;
-			set = &MPD::Song::SetArtist;
-			break;
-		case 2:
-			get = &MPD::Song::GetAlbum;
-			set = &MPD::Song::SetAlbum;
-			break;
-		case 3:
-			get = &MPD::Song::GetDate;
-			set = &MPD::Song::SetDate;
-			break;
-		case 4:
-			get = &MPD::Song::GetTrack;
-			set = &MPD::Song::SetTrack;
-			if (w == TagTypes)
+		LockStatusbar();
+		Statusbar() << "Number tracks? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+		wFooter->Refresh();
+		int in = 0;
+		do
+		{
+			TraceMpdStatus();
+			wFooter->ReadKey(in);
+		}
+		while (in != 'y' && in != 'n');
+		UnlockStatusbar();
+		if (in == 'y')
+		{
+			MPD::SongList::iterator it = EditedSongs.begin();
+			for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
 			{
-				LockStatusbar();
-				Statusbar() << "Number tracks? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
-				wFooter->Refresh();
-				int in = 0;
-				do
-				{
-					TraceMpdStatus();
-					wFooter->ReadKey(in);
-				}
-				while (in != 'y' && in != 'n');
-				UnlockStatusbar();
-				if (in == 'y')
-				{
-					MPD::SongList::iterator it = EditedSongs.begin();
-					for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
-					{
-						if (Config.tag_editor_extended_numeration)
-							(*it)->SetTrack(IntoStr(i) + "/" + IntoStr(EditedSongs.size()));
-						else
-							(*it)->SetTrack(i);
-					}
-					ShowMessage("Tracks numbered!");
-				}
+				if (Config.tag_editor_extended_numeration)
+					(*it)->SetTrack(IntoStr(i) + "/" + IntoStr(EditedSongs.size()));
 				else
-					ShowMessage("Aborted!");
+					(*it)->SetTrack(i);
 			}
-			break;
-		case 5:
-			get = &MPD::Song::GetGenre;
-			set = &MPD::Song::SetGenre;
-			break;
-		case 6:
-			get = &MPD::Song::GetComposer;
-			set = &MPD::Song::SetComposer;
-			break;
-		case 7:
-			get = &MPD::Song::GetPerformer;
-			set = &MPD::Song::SetPerformer;
-			break;
-		case 8:
-			get = &MPD::Song::GetDisc;
-			set = &MPD::Song::SetDisc;
-			break;
-		case 9:
-			get = &MPD::Song::GetComment;
-			set = &MPD::Song::SetComment;
-			break;
-		case 10:
+			ShowMessage("Tracks numbered!");
+		}
+		else
+			ShowMessage("Aborted!");
+		return;
+	}
+	
+	if (id < 11)
+	{
+		MPD::Song::GetFunction get = Info::Tags[id].Get;
+		MPD::Song::SetFunction set = Info::Tags[id].Set;
+		if (id > 0 && w == TagTypes)
+		{
+			LockStatusbar();
+			Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
+			std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
+			UnlockStatusbar();
+			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
+				(*it)->SetTags(set, new_tag);
+		}
+		else if (w == Tags)
+		{
+			LockStatusbar();
+			Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
+			std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
+			UnlockStatusbar();
+			if (new_tag != Tags->Current().GetTags(get))
+				Tags->Current().SetTags(set, new_tag);
+			Tags->Scroll(wDown);
+		}
+	}
+	else
+	{
+		if (id == 11) // filename related options
 		{
 			if (w == TagTypes)
 			{
@@ -609,15 +599,13 @@ void TagEditor::EnterPressed()
 					s.SetNewName(new_name + extension);
 				Tags->Scroll(wDown);
 			}
-			return;
 		}
-		case 11: // reset
+		else if (id == 12) // reset
 		{
 			Tags->Clear();
 			ShowMessage("Changes reset");
-			return;
 		}
-		case 12: // save
+		else if (id == 13) // save
 		{
 			bool success = 1;
 			ShowMessage("Writing changes...");
@@ -644,46 +632,21 @@ void TagEditor::EnterPressed()
 			}
 			else
 				Tags->Clear();
-			return;
 		}
-		case 13: // capitalize first letters
+		else if (id == 14) // capitalize first letters
 		{
 			ShowMessage("Processing...");
 			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
 				CapitalizeFirstLetters(**it);
 			ShowMessage("Done!");
-			break;
 		}
-		case 14: // lower all letters
+		else if (id == 15) // lower all letters
 		{
 			ShowMessage("Processing...");
 			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
 				LowerAllLetters(**it);
 			ShowMessage("Done!");
-			break;
 		}
-		default:
-			break;
-	}
-	
-	if (w == TagTypes && id != 0 && id != 4 && set)
-	{
-		LockStatusbar();
-		Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
-		std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
-		UnlockStatusbar();
-		for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
-			(*it)->SetTags(set, new_tag);
-	}
-	else if (w == Tags && set)
-	{
-		LockStatusbar();
-		Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
-		std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
-		UnlockStatusbar();
-		if (new_tag != Tags->Current().GetTags(get))
-			Tags->Current().SetTags(set, new_tag);
-		Tags->Scroll(wDown);
 	}
 }
 
@@ -884,6 +847,63 @@ void TagEditor::PrevColumn()
 	}
 }
 
+void TagEditor::LocateSong(const MPD::Song &s)
+{
+	if (s.GetDirectory().empty())
+		return;
+	
+	if (LeftColumn == Albums)
+	{
+		Config.albums_in_tag_editor = false;
+		if (w == LeftColumn)
+			w = Dirs;
+		LeftColumn = Dirs;
+	}
+		
+	if (Global::myScreen != this)
+		SwitchTo();
+	
+	// go to right directory
+	if (itsBrowsedDir != s.GetDirectory())
+	{
+		itsBrowsedDir = s.GetDirectory();
+		itsBrowsedDir = itsBrowsedDir.substr(0, itsBrowsedDir.rfind('/'));
+		if (itsBrowsedDir.empty())
+			itsBrowsedDir = "/";
+		Dirs->Clear();
+		Update();
+	}
+	if (itsBrowsedDir == "/")
+		Dirs->Reset(); // go to the first pos, which is "." (music dir root)
+	
+	// highlight directory we need and get files from it
+	std::string dir = ExtractTopDirectory(s.GetDirectory());
+	for (size_t i = 0; i < Dirs->Size(); ++i)
+	{
+		if ((*Dirs)[i].first == dir)
+		{
+			Dirs->Highlight(i);
+			break;
+		}
+	}
+	Tags->Clear();
+	Update();
+	
+	// go to the right column
+	NextColumn();
+	NextColumn();
+	
+	// highlight our file
+	for (size_t i = 0; i < Tags->Size(); ++i)
+	{
+		if ((*Tags)[i].GetHash() == s.GetHash())
+		{
+			Tags->Highlight(i);
+			break;
+		}
+	}
+}
+
 void TagEditor::ReadTags(MPD::Song &s)
 {
 	TagLib::FileRef f(s.GetFile().c_str());
@@ -900,6 +920,7 @@ void TagEditor::ReadTags(MPD::Song &s)
 	s.SetGenre(f.tag()->genre().to8Bit(1));
 	if (mpegf)
 	{
+		s.SetAlbumArtist(!mpegf->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TPE2"].front()->toString().to8Bit(1) : "");
 		s.SetComposer(!mpegf->ID3v2Tag()->frameListMap()["TCOM"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TCOM"].front()->toString().to8Bit(1) : "");
 		s.SetPerformer(!mpegf->ID3v2Tag()->frameListMap()["TOPE"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TOPE"].front()->toString().to8Bit(1) : "");
 		s.SetDisc(!mpegf->ID3v2Tag()->frameListMap()["TPOS"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().to8Bit(1) : "");
@@ -925,6 +946,11 @@ void TagEditor::WriteXiphComments(const MPD::Song &s, TagLib::Ogg::XiphComment *
 	
 	tag->addField("DISCNUMBER", ToWString(s.GetDisc())); // disc
 	
+	tag->removeField("ALBUM ARTIST"); // album artist
+	GetTagList(list, s, &MPD::Song::GetAlbumArtist);
+	for (TagLib::StringList::ConstIterator it = list.begin(); it != list.end(); ++it)
+		tag->addField("ALBUM ARTIST", *it, 0);
+	
 	tag->removeField("COMPOSER"); // composer
 	GetTagList(list, s, &MPD::Song::GetComposer);
 	for (TagLib::StringList::ConstIterator it = list.begin(); it != list.end(); ++it)
@@ -967,6 +993,9 @@ bool TagEditor::WriteTags(MPD::Song &s)
 			WriteID3v2("TCON", tag, ToWString(s.GetGenre()));  // genre
 			WriteID3v2("TPOS", tag, ToWString(s.GetDisc()));   // disc
 			
+			GetTagList(list, s, &MPD::Song::GetAlbumArtist);
+			WriteID3v2("TPE2", tag, list); // album artist
+			
 			GetTagList(list, s, &MPD::Song::GetComposer);
 			WriteID3v2("TCOM", tag, list); // composer
 			
@@ -996,7 +1025,7 @@ bool TagEditor::WriteTags(MPD::Song &s)
 			locale_to_utf(new_name);
 			if (rename(path_to_file.c_str(), new_name.c_str()) == 0 && !file_is_from_db)
 			{
-				if (myOldScreen == myPlaylist)
+				if (Global::myOldScreen == myPlaylist)
 				{
 					// if we rename local file, it won't get updated
 					// so just remove it from playlist and add again
@@ -1026,11 +1055,11 @@ std::string TagEditor::CapitalizeFirstLetters(const std::string &s)
 	if (s.empty())
 		return "";
 	std::string result = s;
-	if (isalpha(result[0]))
+	if (!isspace(result[0]))
 		result[0] = toupper(result[0]);
 	for (std::string::iterator it = result.begin()+1; it != result.end(); ++it)
 	{
-		if (isalpha(*it) && !isalpha(*(it-1)) && *(it-1) != '\'')
+		if (!isspace(*it) && isspace(*(it-1)) && *(it-1) != '\'')
 			*it = toupper(*it);
 	}
 	return result;
@@ -1038,49 +1067,25 @@ std::string TagEditor::CapitalizeFirstLetters(const std::string &s)
 
 void TagEditor::CapitalizeFirstLetters(MPD::Song &s)
 {
-	s.SetTitle(CapitalizeFirstLetters(s.GetTitle()));
-	s.SetArtist(CapitalizeFirstLetters(s.GetArtist()));
-	s.SetAlbum(CapitalizeFirstLetters(s.GetAlbum()));
-	s.SetGenre(CapitalizeFirstLetters(s.GetGenre()));
-	s.SetComposer(CapitalizeFirstLetters(s.GetComposer()));
-	s.SetPerformer(CapitalizeFirstLetters(s.GetPerformer()));
-	s.SetDisc(CapitalizeFirstLetters(s.GetDisc()));
-	s.SetComment(CapitalizeFirstLetters(s.GetComment()));
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	{
+		unsigned i = 0;
+		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
+			(s.*m->Set)(CapitalizeFirstLetters(tag), i);
+	}
 }
 
 void TagEditor::LowerAllLetters(MPD::Song &s)
 {
-	std::string conv = s.GetTitle();
-	ToLower(conv);
-	s.SetTitle(conv);
-	
-	conv = s.GetArtist();
-	ToLower(conv);
-	s.SetArtist(conv);
-	
-	conv = s.GetAlbum();
-	ToLower(conv);
-	s.SetAlbum(conv);
-	
-	conv = s.GetGenre();
-	ToLower(conv);
-	s.SetGenre(conv);
-	
-	conv = s.GetComposer();
-	ToLower(conv);
-	s.SetComposer(conv);
-	
-	conv = s.GetPerformer();
-	ToLower(conv);
-	s.SetPerformer(conv);
-	
-	conv = s.GetDisc();
-	ToLower(conv);
-	s.SetDisc(conv);
-	
-	conv = s.GetComment();
-	ToLower(conv);
-	s.SetComment(conv);
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	{
+		unsigned i = 0;
+		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
+		{
+			ToLower(tag);
+			(s.*m->Set)(tag, i);
+		}
+	}
 }
 
 void TagEditor::GetTagList(TagLib::StringList &list, const MPD::Song &s, MPD::Song::GetFunction f)
@@ -1094,44 +1099,11 @@ void TagEditor::GetTagList(TagLib::StringList &list, const MPD::Song &s, MPD::So
 std::string TagEditor::TagToString(const MPD::Song &s, void *data)
 {
 	std::string result;
-	switch (static_cast<Menu<std::string> *>(data)->Choice())
-	{
-		case 0:
-			result = s.GetTitle();
-			break;
-		case 1:
-			result = s.GetArtist();
-			break;
-		case 2:
-			result = s.GetAlbum();
-			break;
-		case 3:
-			result = s.GetDate();
-			break;
-		case 4:
-			result = s.GetTrack();
-			break;
-		case 5:
-			result = s.GetGenre();
-			break;
-		case 6:
-			result = s.GetComposer();
-			break;
-		case 7:
-			result = s.GetPerformer();
-			break;
-		case 8:
-			result = s.GetDisc();
-			break;
-		case 9:
-			result = s.GetComment();
-			break;
-		case 11:
-			result = s.GetNewName().empty() ? s.GetName() : s.GetName() + " -> " + s.GetNewName();
-			break;
-		default:
-			break;
-	}
+	size_t i = static_cast<Menu<std::string> *>(data)->Choice();
+	if (i < 11)
+		result = (s.*Info::Tags[i].Get)(0);
+	else if (i == 12)
+		result = s.GetNewName().empty() ? s.GetName() : s.GetName() + " -> " + s.GetNewName();
 	return result.empty() ? Config.empty_tag : result;
 }
 
@@ -1169,6 +1141,8 @@ MPD::Song::SetFunction TagEditor::IntoSetFunction(char c)
 	{
 		case 'a':
 			return &MPD::Song::SetArtist;
+		case 'A':
+			return &MPD::Song::SetAlbumArtist;
 		case 't':
 			return &MPD::Song::SetTitle;
 		case 'b':
